varying highp vec2 textureCoordinate;




uniform sampler2D inputImageTexture;


//0: 亮度
//      图像的亮度, 指的是图像像素的强度, 黑色为最暗, 白色为最亮, 在RGB中黑色用0来表示, 白色用1来表示.一个像素, 基本上是用RGB三个颜色分量来表示的. R(0-1), G(0-1),B(0-1).
//      亮度调整有多种计算方法,效果并不完全相同，在颜色的表示方法中， HSL(L)表示法就是：色相(hue)、饱和度(saturation)、亮度(lightness)，改变其中的L值就可以调整图象的亮度，但效果显得比较生硬。
//      PhotoShop和GPUImage中采用的就是另外一种方法就是把图象每个点颜色的RGB分量分别加上亮度调整值，这种效果相对比较柔和。
uniform lowp float brightness; // 亮度值

//1: 对比度
//      对比度指的是一幅图像中明暗区域最亮的白和最暗的黑之间不同亮度层级的测量，差异范围越大代表对比越大，差异范围越小代表对比越小。
//      好的对比率120:1就可容易地显示生动、丰富的色彩，当对比率高达300:1时，便可支持各阶的颜色。
uniform lowp float contrast;   // 对比度

//2: 色温
//      色温是表示光源光谱质量最通用的指标。
uniform lowp float temperature;
uniform lowp float tint;

const lowp vec3 warmFilter = vec3(0.93, 0.54, 0.0);
const mediump mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
const mediump mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);

//3: 饱和度
// 　　  饱和度是指色彩的鲜艳程度，也称色彩的纯度。饱和度取决于该色中含色成分和消色成分（灰色）的比例。含色成分越大，饱和度越大；消色成分越大，饱和度越小。纯的颜色都是高度饱和的.
//       如鲜红，鲜绿。混杂上白色，灰色或其他色调的颜色，是不饱和的颜色，如绛紫，粉红，黄褐等。完全不饱和的颜色根本没有色调，如黑白之间的各种灰色。
uniform lowp float saturation;
const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721); //光亮度里三个值相加要为1，各个值代表着颜色的百分比，中间是绿色的值，70%的比重会让效果更好点。

//4: 锐化
//       应用锐化工具可以快速聚焦模糊边缘，提高图像中某一部位的清晰度或者焦距程度，使图像特定区域的色彩更加鲜明。
uniform lowp float sharpen;

//5: 高光
uniform lowp float highlights;
const mediump vec3 highlightLuminanceWeighting = vec3(0.3, 0.3, 0.3);

////6: 暗部
uniform lowp float shadows;

//7: 智能补光
uniform highp float exposure;





void main()
{
    // 根据坐标取样图片颜色信息
    lowp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);
    
    // 0. 亮度
    lowp vec4 brightnessColor = vec4((textureColor.rgb + vec3(brightness)), textureColor.a);
    
    // 1. 对比度
    lowp vec4 contrastColor = vec4(((brightnessColor.rgb - vec3(0.5)) * contrast + vec3(0.5)), brightnessColor.a);
    
    // 2. 色温
    // 将彩色图像从RGB转换到YIQ色彩空间，可以把彩色图像中的亮度信息与色度信息分开，分别独立进行处理。
    // Y:亮度(图像的灰度值) I:色调  Q:饱和度
    // RGB in the range [0,1] to YIQ in the range Y in [0,1], I in [-0.5957,0.5957], Q in [-0.5226,0.5226].
    mediump vec3 yiq = RGBtoYIQ * contrastColor.rgb;
    yiq.b = clamp(yiq.b + tint * 0.5226 * 0.1, -0.5226, 0.5226);
    lowp vec3 rgb = YIQtoRGB * yiq;
    lowp vec3 processed = vec3(
                               (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))),
                               (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
                               (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));
    lowp vec4 temperatureColor = vec4(mix(rgb, processed, temperature), contrastColor.a);
    
    // 3. 饱和度
    // GLSL中的点乘运算，线性代数的点运算符相乘两个数字。点乘计算需要将纹理颜色信息和相对应的亮度权重相乘。然后取出所有的三个值相加到一起计算得到这个像素的中和亮度值。
    lowp float luminance = dot(temperatureColor.rgb, luminanceWeighting);
    // 创建一个三个值都是亮度信息的vec3
    lowp vec3 greyScaleColor = vec3(luminance);
    // 用mix函数把计算的灰度值，初始的纹理颜色和得到的饱和度信息结合起来。
    lowp vec4 saturationColor = vec4(mix(greyScaleColor, temperatureColor.rgb, saturation), temperatureColor.a);
    
    // 4. 锐化
    
    // 5. 高光
    lowp float highlightLuminance = dot(saturationColor.rgb, highlightLuminanceWeighting);
    
    mediump float shadow = clamp((pow(highlightLuminance, 1.0/(shadows+1.0)) + (-0.76)*pow(highlightLuminance, 1.0/(shadows+1.0))) - highlightLuminance, 0.0, 1.0);
    mediump float highlight = clamp((1.0 - (pow(1.0-highlightLuminance, 1.0/(2.0-highlights)) + (-0.8)*pow(1.0-highlightLuminance, 2.0/(2.0-highlights)))) - highlightLuminance, -1.0, 0.0);
    lowp vec3 result = vec3(0.0, 0.0, 0.0) + ((highlightLuminance + shadow + highlight) - 0.0) * ((saturationColor.rgb - vec3(0.0, 0.0, 0.0))/(highlightLuminance - 0.0));
    lowp vec4 highlightColor = vec4(result, saturationColor.a);
    
    // 7. 智能补光
    lowp vec4 exposureColor = vec4(highlightColor.rgb * pow(2.0, exposure), highlightColor.a);

    gl_FragColor = exposureColor;
}
